# Plugin module for TWiki Enterprise Collaboration Platform, http://TWiki.org/
#
# Copyright (C) 2011-2018 Peter Thoeny, peter[at]thoeny.org
# Copyright (C) 2011-2018 TWiki Contributors. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. For
# more details read LICENSE in the root of this distribution.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

package TWiki::Plugins::QRCodePlugin::Core;

use strict;

require TWiki::Func;    # The plugins API

our @sizeToVersion = (
    1, 14, 16, 42, 62, 84, 106, 122, 152, 180, 213,
    251, 287, 331, 362, 412, 450, 504, 560, 624,
    666, 711, 779, 857, 911, 997, 1059, 1125, 1190, 1264,
    1370, 1452, 1538, 1628, 1722, 1809, 1911, 1989, 2099, 2213, 2331
);

#==========================
sub handleQRCODE {
    my( $session, $params, $theTopic, $theWeb ) = @_;

    my $text   = $params->{_DEFAULT} || '';
    my $pEcc   = $params->{ecc} || 'M';
    my $pVer   = $params->{version} || '8';
    my $pSize  = $params->{size} || '4';
    my $format = $params->{format} ||
                 '<img src="$urlpath" width="$width" height="$height" border="0" alt="" />';

    return "QRCode Plugin Error: QRCode text is missing." unless( $text );

    my $textLength = length($text);
    if( $pVer eq '0' || $pVer !~ /^[0-9]+$/ ) {
       my $i = 0;
       my $max = (scalar @sizeToVersion) - 1;
       $pVer = $sizeToVersion[$max];
       while( $i++ <= $max && $textLength > $sizeToVersion[$i] ) {
       }
       $pVer = $i;
    }

    # generate image
    require GD::Barcode::QRcode;
    my $qrcode;
    my $image;
    eval {
        $qrcode = GD::Barcode::QRcode->new( $text, {ECC => $pEcc, Version => $pVer, ModuleSize => $pSize} );
        $image = $qrcode->plot->png;
    };
    my $err = $@ || '';
    $err =~ s/[\r\n\t]/ /go;
    return "QRCode Plugin Error: $err" if( $err );

    # save image file
    my( $dir, $fileName ) = _makeFilename( $theWeb, $theTopic, "$text-$pEcc-$pVer-$pSize" );
    open( PNG, "> $dir/$fileName" )
        or return "QRCode Plugin Error: Can't write temporary file $fileName.";
    binmode( PNG );
    print PNG $image;
    close( PNG );

    # generate and return HTML image tag
    my $maxModules = $qrcode->{MaxModules} || 17 + ( $pVer * 4 ); # use undocumented || hard-code
    my $pixels = ( $maxModules + 8 ) * $pSize;
    my $urlPath = TWiki::Func::getPubUrlPath() . "/$theWeb/$theTopic/$fileName";
    $format =~ s/\$urlpath/$urlPath/;
    $format =~ s/\$width/$pixels/;
    $format =~ s/\$height/$pixels/;
    return $format;
}

#==========================
sub _makeFilename
{
    my ( $web, $topic, $text ) = @_;

    my $dir = TWiki::Func::getPubDir() . "/$web";
    unless( -e "$dir" ) {
        umask( 002 );
        mkdir( $dir, 0775 );
    }
    $dir .= "/$topic";
    unless( -e "$dir" ) {
        umask( 002 );
        mkdir( $dir, 0775 );
    }

    use Digest::MD5  qw(md5_hex);
    my $md5 = md5_hex( $text );

    my $name = "_QRCodePlugin_$md5.png";

    return( $dir, $name );
}

#==========================
1;
